#include "xfuncs.h"

void init_widgets(GameControl** gamec) {

	open_map_window(gamec);

	/* starting the game loop */

	g_timeout_add(GAME_CYCLE_TIME,(GtkFunction)do_cycle,gamec);

}

void open_map_window(GameControl** gamec) {

	MAP_WINDOW = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(MAP_WINDOW),QUEST_WINDOW_TITLE);
	gtk_container_set_border_width(GTK_CONTAINER(MAP_WINDOW),0);

	gtk_widget_set_events(MAP_WINDOW,GDK_KEY_PRESS_MASK    | GDK_KEY_RELEASE_MASK | 
	                                 GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK );
	gtk_signal_connect(GTK_OBJECT(MAP_WINDOW),"delete_event",GTK_SIGNAL_FUNC(delete_event),NULL); 
	gtk_signal_connect(GTK_OBJECT(MAP_WINDOW),"destroy",GTK_SIGNAL_FUNC(destroy),NULL);
	gtk_signal_connect(GTK_OBJECT(MAP_WINDOW),"key_press_event",GTK_SIGNAL_FUNC(key_interface),gamec);
	gtk_signal_connect(GTK_OBJECT(MAP_WINDOW),"key_release_event",GTK_SIGNAL_FUNC(key_interface),gamec);
	
	MAP_BOX = gtk_vbox_new(FALSE,0);

	get_map_menu(gamec);
	gtk_box_pack_start(GTK_BOX(MAP_BOX),MAP_MENU,FALSE,TRUE,0);
	gtk_widget_show(MAP_MENU);

	MAP_EVENT_BOX = gtk_event_box_new();
	gtk_box_pack_start(GTK_BOX(MAP_BOX),MAP_EVENT_BOX,TRUE,TRUE,0);
	gtk_widget_show(MAP_EVENT_BOX);

	MAP_DRAWING_AREA = gtk_drawing_area_new();
	gtk_drawing_area_size(GTK_DRAWING_AREA(MAP_DRAWING_AREA), 360, 360);
	gtk_widget_show(MAP_DRAWING_AREA);

	gtk_signal_connect(GTK_OBJECT(MAP_DRAWING_AREA),"configure_event",GTK_SIGNAL_FUNC(configure_event),gamec);
	gtk_signal_connect_after(GTK_OBJECT(MAP_DRAWING_AREA),"expose_event",GTK_SIGNAL_FUNC(expose_event),gamec);
	gtk_signal_connect(GTK_OBJECT(MAP_EVENT_BOX),"button_press_event",GTK_SIGNAL_FUNC(mouse_interface),gamec);
	gtk_signal_connect(GTK_OBJECT(MAP_EVENT_BOX),"button_release_event",GTK_SIGNAL_FUNC(mouse_interface),gamec);
	gtk_container_add(GTK_CONTAINER(MAP_EVENT_BOX),MAP_DRAWING_AREA);

	gtk_container_add(GTK_CONTAINER(MAP_WINDOW),MAP_BOX);
	gtk_widget_show(MAP_BOX);
	gtk_widget_show(MAP_WINDOW);

}

void open_inv_window(GameControl** gamec) {

	gchar* inv_clist_titles[5] = {"","item name","weight","gold",""};

	if(INV_WINDOW == NULL) {

		INV_WINDOW = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		gtk_window_set_title(GTK_WINDOW(INV_WINDOW),"Inventory Screen");
		gtk_container_set_border_width(GTK_CONTAINER(INV_WINDOW),0);
		gtk_signal_connect(GTK_OBJECT(INV_WINDOW),"delete_event",GTK_SIGNAL_FUNC(close_inv_window),gamec); 	
		INV_SCROLLED_WIN = gtk_scrolled_window_new(NULL,NULL);
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(INV_SCROLLED_WIN),
		                               GTK_POLICY_AUTOMATIC,GTK_POLICY_ALWAYS);
		gtk_widget_set_usize(GTK_WIDGET(INV_SCROLLED_WIN), 275, 200);
	
		INV_BOX = gtk_vbox_new(FALSE,0);
	
		get_inv_menu(gamec);
		gtk_box_pack_start(GTK_BOX(INV_BOX),INV_MENU,FALSE,TRUE,0);
		gtk_widget_show(INV_MENU);
		gtk_box_pack_start(GTK_BOX(INV_BOX),INV_SCROLLED_WIN,FALSE,TRUE,0);
		gtk_widget_show(INV_SCROLLED_WIN);
	
		INV_CLIST = gtk_clist_new_with_titles(5,inv_clist_titles);
		gtk_clist_set_column_width(GTK_CLIST(INV_CLIST), 0, 25);
		gtk_clist_set_column_width(GTK_CLIST(INV_CLIST), 1, 125);
		gtk_clist_set_column_width(GTK_CLIST(INV_CLIST), 2, 35);
		gtk_clist_set_column_width(GTK_CLIST(INV_CLIST), 3, 35);
		gtk_container_add(GTK_CONTAINER(INV_SCROLLED_WIN), INV_CLIST);
		gtk_widget_show(INV_CLIST);
	
		gtk_container_add(GTK_CONTAINER(INV_WINDOW),INV_BOX);
		gtk_widget_show(INV_BOX);
		gtk_widget_show(INV_WINDOW);

	}

}

void close_inv_window(GtkWidget *widget,GdkEvent* event,GameControl** gamec) {

	INV_WINDOW = NULL;

	gtk_widget_destroy(widget);


}

void print_image_list(GameControl** gamec) {
	
	GList* walker = (*gamec)->image_files;
	gint i=0;

	g_print("image list:\n");

	while(walker != NULL) {

		if(walker->data != NULL) {
			
			g_print("%d: %s\n",i,((gchar*)walker->data));

		} else {

			g_print("aargh! there's a NULL node in my image list!\n");

		}

		i++;
		walker = walker->next;

	}

}

void print_monster_list(GameControl** gamec) {
	
	GList* walker = (*gamec)->monsters;
	gint i=0;

	g_print("monster list:\n");

	while(walker != NULL) {

		if(walker->data != NULL) {
			
			g_print("%d: %s %s\n",i,((Monster*)walker->data)->name->str,
			                      (CHK(((Monster*)walker->data)->properties,IS_PLAYER))?"(p)":"");

		} else {

			g_print("aargh! there's a NULL node in my monster list!\n");

		}

		i++;
		walker = walker->next;

	}

}

void print_item_list(GameControl** gamec) {
	
	GList* walker = (*gamec)->items;
	gint i=0;

	g_print("item list:\n");

	while(walker != NULL) {

		if(walker->data != NULL) {
			
			g_print("%d: %s\n",i,((Item*)walker->data)->name->str);

		} else {

			g_print("aargh! there's a NULL node in my item list!\n");

		}

		i++;
		walker = walker->next;

	}

}

void print_key_stack(GameControl** gamec) {
	
	GList* walker = (*gamec)->key_stack;
	gint i=0;

	g_print("key stack:\n");

	while(walker != NULL) {

		if(walker->data != NULL) {
			
			g_print("%d: %d (%s)\n",i,(gint)walker->data,gdk_keyval_name((gint)walker->data));

		} else {

			g_print("aargh! there's a NULL node in my item stack!\n");

		}

		i++;
		walker = walker->next;

	}

}

void print_cycle_cnt(GameControl** gamec) {

	g_print("%d\n",CYCLE_CNT);
}

void get_map_menu(GameControl** gamec) {

	GtkItemFactory *item_factory;
	GtkAccelGroup *accel_group;
	GtkItemFactoryEntry menu_items[] = {
		{"/_Game",NULL,NULL,0,"<Branch>"},
		{"/Game/Quit","<control>Q",gtk_main_quit,0,NULL},
		{"/_Debug",NULL,NULL,0,"<Branch>"},
		{"/Debug/Print Cycle",NULL,print_cycle_cnt,0,NULL},
		{"/Debug/Print Image List",NULL,print_image_list,0,NULL},
		{"/Debug/Print Monster List",NULL,print_monster_list,0,NULL},
		{"/Debug/Print Item List",NULL,print_item_list,0,NULL},
		{"/Debug/Print Key Stack",NULL,print_key_stack,0,NULL},
		{"/Debug/A Different Point of View",NULL,change_player,0,NULL}, /* definitely remove this one soon */
		{"/_Commands",NULL,NULL,0,"<Branch>"},
		{"/Commands/(un)Freeze Game","<control>P",freeze_game,0,NULL},
		{"/Commands/(un)Freeze Monsters","<control><alt>P",freeze_monsters,0,NULL},
		{"/_Windows",NULL,NULL,0,"<Branch>"},
		{"/Windows/Inventory","<control>I",open_inv_window,0,NULL},
		{"/_Help",NULL,NULL,0,"<LastBranch>"},
		{"/_Help/About",NULL,about,0,NULL}
	};
	gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);

	accel_group = gtk_accel_group_new ();
	item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR,"<main>",accel_group);

	gtk_item_factory_create_items(item_factory,nmenu_items,menu_items,gamec);
	gtk_accel_group_attach(accel_group, GTK_OBJECT(MAP_WINDOW));
	
	MAP_MENU = gtk_item_factory_get_widget(item_factory,"<main>");

}

void get_inv_menu(GameControl** gamec) {

	GtkItemFactory *item_factory;
	GtkAccelGroup *accel_group;
	GtkItemFactoryEntry menu_items[] = {
		{"/_Game",NULL,NULL,0,"<Branch>"},
		{"/Game/Quit","<control>Q",gtk_main_quit,0,NULL},
		{"/_Help",NULL,NULL,0,"<LastBranch>"},
		{"/_Help/About",NULL,about,0,NULL}
	};
	gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);

	accel_group = gtk_accel_group_new ();
	item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR,"<main>",accel_group);

	gtk_item_factory_create_items(item_factory,nmenu_items,menu_items,gamec);
	gtk_accel_group_attach(accel_group, GTK_OBJECT(INV_WINDOW));
	
	INV_MENU = gtk_item_factory_get_widget(item_factory,"<main>");

}

gint delete_event(GtkWidget *widget,GdkEvent *event,gpointer data) {

	/* turn TRUE into FALSE and the window will get destroyed */
	return FALSE;

}

gint expose_event(GtkWidget *widget,GdkEventExpose *event,GameControl** gamec) {

	gint x,y;

	if(event->send_event == FALSE) { /* == if the expose isn't synthesized by gtk_widget_draw() */

		(*gamec)->game_state = SET((*gamec)->game_state,GAME_FREEZE);

		for(y = (AREA_OY+(event->area.y/TILE_SIZE));y<=(AREA_OY+((event->area.y+event->area.height)/TILE_SIZE));y++)
		for(x = (AREA_OX+(event->area.x/TILE_SIZE));x<=(AREA_OX+((event->area.x+event->area.width)/TILE_SIZE));x++) {

			add_update(gamec,x,y);

		}

		(*gamec)->game_state = CLR((*gamec)->game_state,GAME_FREEZE);
		

	}

	return TRUE;

}


gint configure_event(GtkWidget *widget,GdkEventConfigure *event,GameControl** gamec) {

	gint x,y;

	if(MAP_AREA != NULL) {

		gdk_pixmap_unref(MAP_AREA);

	} 

	MAP_AREA = gdk_pixmap_new(widget->window,widget->allocation.width,widget->allocation.height,-1);
	
	for(y = AREA_OY;y<=(AREA_OY+(widget->allocation.height/TILE_SIZE));y++)
	for(x = AREA_OX;x<=(AREA_OX+(widget->allocation.width/TILE_SIZE));x++) {

		add_update(gamec,x,y);

	}

	return TRUE;

}
            
void destroy(GtkWidget *widget,gpointer data) {

	gtk_main_quit();

}

void about(GameControl** gamec) {

	GtkWidget* about;
	GtkWidget* button;
	GtkWidget* label;
	GtkWidget* pixmap1;
	GtkWidget* pixmap2;
	GtkWidget* hbox;

	about = gtk_dialog_new();
	gtk_widget_show(about);

	hbox = gtk_hbox_new(FALSE,0);
	gtk_widget_ref (hbox);
	gtk_object_set_data_full(GTK_OBJECT(about),"hbox",hbox,(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(hbox);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(about)->vbox),hbox,TRUE, TRUE, 0);

	pixmap1 = gtk_pixmap_new(PIXMAP(OGRE_DOWN_WALK_RIGHT_1),MASK(OGRE_DOWN_WALK_RIGHT_1));
	gtk_widget_ref(pixmap1);
	gtk_object_set_data_full(GTK_OBJECT(about),"pixmap1",pixmap1,(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(pixmap1);
	gtk_box_pack_start(GTK_BOX(hbox),pixmap1,TRUE,TRUE,0);
	gtk_misc_set_alignment(GTK_MISC (pixmap1),1,0.5);

	pixmap2 = gtk_pixmap_new(PIXMAP(KNIGHT_DOWN_WALK_LEFT_1),MASK(KNIGHT_DOWN_WALK_LEFT_1));
	gtk_widget_ref(pixmap2);
	gtk_object_set_data_full(GTK_OBJECT(about),"pixmap2",pixmap2,(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show(pixmap2);
	gtk_box_pack_start(GTK_BOX(hbox),pixmap2,TRUE,TRUE,0);
	gtk_misc_set_alignment(GTK_MISC(pixmap2),0,0.5);

	label = gtk_label_new(ABOUT_STRING);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(about)->vbox),label,FALSE,FALSE,0);
	gtk_misc_set_padding(GTK_MISC(label),30,5);
	gtk_widget_show(label);

	button = gtk_button_new_with_label("Ok");
	gtk_signal_connect(GTK_OBJECT(button),"clicked",GTK_SIGNAL_FUNC(about_close),GTK_WIDGET(about));
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(about)->action_area),button,TRUE,TRUE,0);
	gtk_widget_show(button);
}

void about_close(GtkWidget *widget,gpointer data) {
	
	gtk_widget_destroy(GTK_WIDGET(data));

}

MapWin* new_map_window() {
	
	MapWin* new = (MapWin*)g_malloc(sizeof(MapWin));

	new->window = NULL;
	new->box = NULL;
	new->menu = NULL;
	new->drawing_area = NULL;
	new->event_box = NULL;
	new->area = NULL;
	new->gc = NULL;
	new->update = NULL;
	new->area_ox = 0;
	new->area_oy = 0;

	return new;

}

void zap_map_window(MapWin* map_window) {

	if(map_window->update != NULL) {

		g_list_foreach(map_window->update,(GFunc)zap_coords,NULL);
		g_list_free(map_window->update);
		
	}

	gdk_pixmap_unref(map_window->area);
	gdk_gc_unref(map_window->gc);

	if(map_window != NULL) {

		g_free(map_window);
		map_window = NULL;

	}

}

InvWin* new_inv_window() {
	
	InvWin* new = (InvWin*)g_malloc(sizeof(InvWin));

	new->window = NULL;
	new->scrolled_win = NULL;
	new->box = NULL;
	new->menu = NULL;
	new->clist = NULL;
	new->who = NULL;

	return new;

}

void zap_inv_window(InvWin* inv_window) {

	inv_window->who = NULL;

	if(inv_window != NULL) {

		g_free(inv_window);
		inv_window = NULL;

	}

}

void display_contains(GameControl** gamec,Monster* who) {

	GList* walker = who->contains;
	gint cnt = 0;
	gchar *text[5],inv_key[4] = {0},inv_weight[6] = {0},inv_value[8] = {0};


	gtk_clist_clear((GtkCList*)INV_CLIST);	
	INV_WHO = who;

	while(walker != NULL) {

		if(walker->data != NULL) {

			inv_key[0] = (((Item*)walker->data)->key/100 == 0)
			             ?((((Item*)walker->data)->key>99)?'0':' ')
			             :(((Item*)walker->data)->key/100)%10+'0';
			inv_key[1] = (((Item*)walker->data)->key/10 == 0)
			             ?((((Item*)walker->data)->key>9)?'0':' ')
			             :(((Item*)walker->data)->key/10)%10+'0';
			inv_key[2] = ((Item*)walker->data)->key%10+'0';
			inv_key[3] = '\0';

			cnt+=((Item*)walker->data)->weight;

			inv_weight[0] = (((Item*)walker->data)->weight/10000 == 0)
			                ?((((Item*)walker->data)->weight>9999)?'0':' ')
			                :(((Item*)walker->data)->weight/10000)%10+'0';
			inv_weight[1] = (((Item*)walker->data)->weight/1000 == 0)
			                ?((((Item*)walker->data)->weight>999)?'0':' ')
			                :(((Item*)walker->data)->weight/1000)%10+'0';
			inv_weight[2] = (((Item*)walker->data)->weight/100 == 0)
			                ?((((Item*)walker->data)->weight>99)?'0':' ')
			                :(((Item*)walker->data)->weight/100)%10+'0';
			inv_weight[3] = (((Item*)walker->data)->weight/10 == 0)
			                ?((((Item*)walker->data)->weight>9)?'0':' ')
			                :(((Item*)walker->data)->weight/10)%10+'0';
			inv_weight[4] = ((Item*)walker->data)->weight%10+'0';
			inv_weight[5] = '\0';

			inv_value[0] = (((Item*)walker->data)->value/100000 == 0)
			               ?((((Item*)walker->data)->value>99999)?'0':' ')
			               :(((Item*)walker->data)->value/100000)%10+'0';
			inv_value[1] = (((Item*)walker->data)->value/10000 == 0)
			               ?((((Item*)walker->data)->value>9999)?'0':' ')
			               :(((Item*)walker->data)->value/10000)%10+'0';
			inv_value[2] = (((Item*)walker->data)->value/1000 == 0)
			               ?((((Item*)walker->data)->value>999)?'0':' ')
			               :(((Item*)walker->data)->value/1000)%10+'0';
			inv_value[3] = (((Item*)walker->data)->value/100 == 0)
			               ?((((Item*)walker->data)->value>99)?'0':' ')
			               :(((Item*)walker->data)->value/100)%10+'0';
			inv_value[4] = (((Item*)walker->data)->value%100==0)?' ':'.';
			inv_value[5] = (((Item*)walker->data)->value%100==0)?' ':
			               (((Item*)walker->data)->value/10 == 0)
			               ?((((Item*)walker->data)->value>9)?'0':' ')
			               :(((Item*)walker->data)->value/10)%10+'0';
			inv_value[6] = (((Item*)walker->data)->value%10==0)?' ':
			               ((Item*)walker->data)->value%10+'0';
			inv_value[7] = '\0';

			text[0] = inv_key;
			text[1] = ((Item*)walker->data)->name->str;
			text[2] = inv_weight;
			text[3] = inv_value;
			text[4] = "";

			gtk_clist_append((GtkCList*)INV_CLIST,text);

		} else {

			g_print("there's NULL data in the contains of %s!\n",who->name->str);

		}

		walker = walker->next;

	}
	
	inv_weight[0] = (cnt/10000 == 0)?((cnt>9999)?'0':' ')
	                :(cnt/10000)%10+'0';
	inv_weight[1] = (cnt/1000 == 0)?((cnt>999)?'0':' ')
	                :(cnt/1000)%10+'0';
	inv_weight[2] = (cnt/100 == 0)?((cnt>99)?'0':' ')
	                :(cnt/100)%10+'0';
	inv_weight[3] = (cnt/10 == 0)?((cnt>9)?'0':' ')
	                :(cnt/10)%10+'0';
	inv_weight[4] = (cnt%10)+'0';
	inv_weight[5] = '\0';

	text[0] = "";
	text[1] = "";
	text[2] = inv_weight;
	text[3] = "";
	text[4] = "";

	gtk_clist_append((GtkCList*)INV_CLIST,text);

}

